#include "gdraw.h"
#include "functions.h"

static GDraw *sharedgdraw = nil;

@implementation GDraw

+ (GDraw *)sharedgdraw
{
	if(!sharedgdraw) {
      NS_DURING
      {
	 	 	[gnustep_global_lock lock];
	  		sharedgdraw = [[self alloc] init];
	  		[gnustep_global_lock unlock];
		}
		NS_HANDLER
		{
	  		[gnustep_global_lock unlock];
	  		[localException raise];
		}
      NS_ENDHANDLER
	}
	return sharedgdraw;
}

- (void)dealloc
{
	[wins release];
	[tools release];
	[workPath release];
	[pfbPathForFont release];
	[super dealloc];
}

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
	NSArray *fontsDirs;
	NSString *fdpath;
	
	workPath = [[NSString alloc] initWithString: NSHomeDirectory()];
	fdpath = [[[NSBundle mainBundle] bundlePath] stringByAppendingString:@"/Resources/fontsPaths"];
	fontsDirs = [NSArray arrayWithContentsOfFile: fdpath];
	pfbPathForFont = [[NSDictionary alloc] initWithDictionary: pfbFilesInFontsDirs(fontsDirs)];
	wins = [[NSMutableArray alloc] initWithCapacity: 1];
	tools = [[GDToolsWindow alloc] init];
	[tools display];
	[tools orderFront:nil];
}

- (BOOL)applicationShouldTerminate:(NSApplication *)app 
{
	DocWin *win;
	int i, result;

	for(i = 0; i < [wins count]; i++) {
		win = [wins objectAtIndex: i];
		if(![win isSaved]) {
			[win orderFront: nil];
			result = NSRunAlertPanel(@"Alert", 
					[NSString stringWithFormat:@"%@ is not saved! Do you want to save it?", [win name]], 
										@"Ok", @"No", @"Cancel");
			if(result == NS_ALERTDEFAULT) {
				[[win docView] doSave];
				i--;
			} else if(result == NS_ALERTOTHER) {
				return NO;			
			}
		}
	}

	return YES;
}

- (BOOL)application:(NSApplication *)app openFile:(NSString *)filename
{
	if([self loadDocument: filename]) {
		[workPath release];
		workPath = [[NSString alloc] initWithString: [filename stringByDeletingLastPathComponent]];
		return YES;
	}
	return NO;
}

- (void)setToolType:(ToolType)type
{
	tooltype = type;
	[tools setButtonsPositions: tooltype];
}

- (ToolType)currentToolType
{
	return tooltype;
}

- (void)setCurrentView:(DocView *)aView
{
	currentView = aView;
}

- (void)updateCurrentWindow
{
	[[currentView window] display];
}

- (BOOL)removeWinFromWinList:(DocWin *)awin
{
	int result;
	
	if(![awin isSaved]) {
		result = NSRunAlertPanel(@"Alert", 
				[NSString stringWithFormat:@"%@ is not saved! Do you want to save it?", [awin name]], 
								@"Ok", @"No", @"Cancel");
		if(result == NS_ALERTDEFAULT) {
			[[awin docView] doSave];
			if([awin isSaved])
				return YES;
			else
				return NO;
		} else if(result == NS_ALERTALTERNATE) {
			[wins removeObject: awin];
			return YES;
		} else if(result == NS_ALERTOTHER) {
			return NO;
		}
	} else {
		[wins removeObject: awin];
		return YES;
	}
	
	return NO;
}

- (BOOL)loadDocument:(NSString *)docPath
{
	DocWin *docWin;
	int i;
	
	for(i = 0; i < [wins count]; i++) {
		docWin = [wins objectAtIndex: i];
		if([[[docWin docView] docName] isEqualToString: docPath]) {
			NSRunAlertPanel(@"Alert", [NSString stringWithFormat:
				@"%@ is already open!", [docPath lastPathComponent]], @"Ok", nil, nil);
			return NO;
		}
	}
	
	docWin = [[DocWin alloc] init];
	[docWin center];
	[docWin display];
	[docWin orderFront:nil];
	
	if(![[docWin docView] createObjectsFromDictionaryAtPath: docPath]) {
		NSRunAlertPanel(@"Alert", @"Error in reading the file!", @"Ok", nil, nil);
		[wins addObject: docWin];
		[docWin release];
		return NO;
	}

	[wins addObject: docWin];
	[docWin release];
	return YES;
}

- (void)new:(id)sender
{
	DocWin *docWin;
			
	docWin = [[DocWin alloc] init];
	[docWin center];
	[docWin display];
	[docWin orderFront:nil];
	
	[wins addObject: docWin];
	[docWin release];
}

- (void)open:(id)sender
{
	NSOpenPanel *openPanel;
	NSArray *fileTypes;
	NSString *docName;
	int result;

	fileTypes = [NSArray arrayWithObjects: @"gdr", nil];
	openPanel = [NSOpenPanel openPanel];
	[openPanel setTitle: @"open"];	
  	[openPanel setAllowsMultipleSelection: NO];
  	[openPanel setCanChooseFiles: YES];
  	[openPanel setCanChooseDirectories: NO];
  	result = [openPanel runModalForDirectory: workPath file: nil types: fileTypes];
	if(result != NSOKButton)
		return;
	docName = [NSString stringWithString: [openPanel filename]];
	[self loadDocument: docName];
}

- (void)save:(id)sender
{
	[currentView doSave];
}

- (void)saveas:(id)sender
{
	[currentView doSaveAs];
}

- (void)psprint:(id)sender
{
	[currentView doPrint];
}

- (void)cut:(id)sender
{
	[currentView doCut];
}

- (void)copy:(id)sender
{
	[currentView doCopy];
}

- (void)paste:(id)sender
{
	[currentView doPaste];
}

- (void)doUndo:(id)sender
{
	[currentView doUndo];
}

- (void)doRedo:(id)sender
{
	[currentView doRedo];
}

- (void)editObjectsProperties:(id)sender
{
	[currentView editObjectsProperties];
}

- (void)bringToFront:(id)sender
{
	[currentView moveSelectedObjectsToFront];
}

- (void)sendToBack:(id)sender
{
	[currentView moveSelectedObjectsToBack];
}

- (void)method:menuCell
{

}

- (void)setWorkPath:(NSString *)apath
{
	[workPath release];
	workPath = [[NSString alloc] initWithString: apath];
}

- (NSString *)workPath
{
	return workPath;
}

- (NSString *)pfaDescriptionOfFont:(NSString *)fontName
{
	NSString *pfbPath, *pfaData = nil;

	pfbPath = [pfbPathForFont objectForKey: fontName];
	if(pfbPath) {
		if([[pfbPath pathExtension] isEqualToString: @"pfa"]) 
			pfaData = [NSString stringWithContentsOfFile: pfbPath];
		else 
			pfaData = [NSString stringWithString: pfaDataOfBinaryFontAtPath(pfbPath)];
	}
	
	return pfaData;
}

@end














